# Python standard library
from math import sqrt
from typing import List

# Dependencies
from typing_extensions import Self

# ft-euler
from ft_euler.geometry.Vector2 import Vector2
from ft_euler.geometry.Line import Line


class Triangle:
    def __init__(self, point1: Vector2, point2: Vector2, point3: Vector2) -> None:
        self.points = [point1, point2, point3]

    def __str__(self) -> str:
        return str(self.points[0]) + ", " + str(self.points[1]) + ", " + str(self.points[2])

    def to_sides(self) -> List[Line]:
        side1 = Line(self.points[0], self.points[1])
        side2 = Line(self.points[1], self.points[2])
        side3 = Line(self.points[2], self.points[0])

        return [side1, side2, side3]

    def calculate_side_lengths(self) -> list:
        side_length_1 = Vector2.distance(self.points[0], self.points[1])
        side_length_2 = Vector2.distance(self.points[1], self.points[2])
        side_length_3 = Vector2.distance(self.points[2], self.points[0])

        return [side_length_1, side_length_2, side_length_3]

    def calculate_area(self) -> float:
        side_lengths = self.calculate_side_lengths()
        semiperimeter = 0.5 * (side_lengths[0] + side_lengths[1] + side_lengths[2])

        # Calculate area using Heron's formula
        area = sqrt(
            semiperimeter *
            (semiperimeter - side_lengths[0]) *
            (semiperimeter - side_lengths[1]) *
            (semiperimeter - side_lengths[2])
        )

        return area

    def __eq__(self, other: Self) -> bool:
        # Two triangles are equal if points are the same

        case1 = (self.points[0] == other.points[0] and self.points[1] == other.points[1] and
                 self.points[2] == other.points[3])

        case2 = (self.points[0] == other.points[1] and self.points[1] == other.points[2] and
                 self.points[2] == other.points[0])

        case3 = (self.points[0] == other.points[0] and self.points[1] == other.points[1] and
                 self.points[2] == other.points[3])

        if case1 or case2 or case3:
            return True
        else:
            return False
