# Python standard library
from math import pi, atan2

# ft-euler
from ft_euler.geometry.Line import Line
from ft_euler.geometry.Vector2 import Vector2

def pi_2_pi(angle: float) -> float:
    output_angle = (angle + pi) % (2 * pi) - pi
    return output_angle

def quaternion_to_yaw(q) -> float:
    x, y, z, w = q.x, q.y, q.z, q.w
    
    t3 = +2.0 * (w * z + x * y)
    t4 = +1.0 - 2.0 * (y * y + z * z)
    yaw_z = atan2(t3, t4)

    return pi_2_pi(yaw_z)

def point_in_circle(point: Vector2, circle_centre: Vector2, circle_radius: float) -> bool:
    return Vector2.distance(point, circle_centre) <= circle_radius
    
def point_is_on_left_side_of_line(point: Vector2, line: Line) -> bool:
    determinant = (line.p2.x - line.p1.x) * (point.y - line.p1.y) - (point.x - line.p1.x) * (line.p2.y - line.p1.y)
        
    if determinant > 0:
        return False  # Point is on the right side of the line
    elif determinant < 0:
        return True  # Point is on the left side of the line
    else:
        return True # Point is on the line
