# Python standard library
from typing import Tuple

# Dependencies
from typing_extensions import Self


class Vector2:
    def __init__(self, x: float, y: float) -> None:
        self.x = x
        self.y = y

    def __str__(self) -> str:
        return "(" + str(self.x) + ", " + str(self.y) + ")"

    @staticmethod
    def from_float_list(float_list: Tuple[float, float]) -> any:
        return Vector2(float_list[0], float_list[1])

    def to_float_list(self) -> list:
        return [self.x, self.y]

    @staticmethod
    def distance(v1: Self, v2: Self) -> float:
        dx = v1.x - v2.x
        dy = v1.y - v2.y
        return (dx ** 2 + dy ** 2) ** 0.5

    @staticmethod
    def midpoint(v1: Self, v2: Self) -> Self:
        mid_x = (v1.x + v2.x) / 2
        mid_y = (v1.y + v2.y) / 2
        return Vector2(mid_x, mid_y)

    def __eq__(self, other: Self) -> bool:
        return self.x == other.x and self.y == other.y
