# Python standard library
from typing import List

# Dependencies
from scipy.spatial import Delaunay

# ft-euler
from ft_euler.geometry.Vector2 import Vector2
from ft_euler.geometry.Triangle import Triangle
from ft_euler.pathfinding.Track import Track


def tessellate_track(track: Track) -> List[Triangle]:
    culled_tessellated_triangles = []
    relevant_cones = track.get_blue_cones() + track.get_yellow_cones()
    if len(relevant_cones) >= 3:
        tessellated_triangles = []
        relevant_vertices = track.to_np_array_of_pos_lists(include_big_orange=False, include_orange=False, include_car_start=False)
        delaunay = Delaunay(relevant_vertices)
        for np_triangle in delaunay.simplices:
            p1 = relevant_vertices[np_triangle][0]
            p1 = Vector2(p1[0], p1[1])
            p2 = relevant_vertices[np_triangle][1]
            p2 = Vector2(p2[0], p2[1])
            p3 = relevant_vertices[np_triangle][2]
            p3 = Vector2(p3[0], p3[1])
            tessellated_triangles.append(Triangle(p1, p2, p3))
    
        culled_tessellated_triangles = []
    
        for triangle in tessellated_triangles:
            blue_count = 0
            yellow_count = 0
            for point in triangle.points:
                for cone in relevant_cones:
                    if point == cone.pos:
                        if cone.color == "blue":
                            blue_count = blue_count + 1
                        elif cone.color == "yellow":
                            yellow_count = yellow_count + 1
    
            if blue_count < 3 and yellow_count < 3:
                culled_tessellated_triangles.append(triangle)

    return culled_tessellated_triangles
