import unittest
from math import isclose  # Floating point is imprecise
from ft_euler.geometry.Vector2 import Vector2
from ft_euler.geometry.Line import Line


class TestLine(unittest.TestCase):

    def setUp(self):
        self.point1 = Vector2(0, 0)
        self.point2 = Vector2(0, 4)
        self.point3 = Vector2(1, 1)
        self.point4 = Vector2(4, 5)
        self.point5 = Vector2(1, 0)

        self.line = Line(self.point1, self.point2)  # x = 0
        self.line2 = Line(self.point1, self.point3)  # y = x
        self.line3 = Line(self.point3, self.point4)  # y = 4/3x - 1/3
        self.line4 = Line(self.point5, self.point3)  # x = 1
        self.line5 = Line(self.point4, self.point3)  # y = 4/3x - 1/3 (=line3)

    def test_calculate_length(self):
        expected_length1 = 4.0
        actual_length1 = self.line.calculate_length()

        self.assertTrue(isclose(expected_length1, actual_length1))

        expected_length2 = 5.0
        actual_length2 = self.line3.calculate_length()

        self.assertTrue(isclose(expected_length2, actual_length2))

    def test_calculate_slope(self):
        expected_slope1 = float("inf")
        actual_slope1 = self.line.calculate_slope()

        self.assertTrue(isclose(expected_slope1, actual_slope1))

        expected_slope2 = 4 / 3
        actual_slope2 = self.line3.calculate_slope()

        self.assertTrue(isclose(expected_slope2, actual_slope2))

    def test_calculate_y_intercept(self):
        expected_y_intercept1 = None
        actual_y_intercept1 = self.line.calculate_y_intercept()

        self.assertEqual(expected_y_intercept1, actual_y_intercept1, "Actual != Expected")

        expected_y_intercept2 = -1 / 3
        actual_y_intercept2 = self.line3.calculate_y_intercept()

        self.assertTrue(isclose(expected_y_intercept2, actual_y_intercept2))

    def test_angle_between_two_lines(self):
        expected_angle1 = 0.14189
        actual_angle1 = Line.angle_between_two_lines(self.line2, self.line3)

        self.assertTrue(isclose(expected_angle1, actual_angle1, rel_tol=1e-04))

        expected_angle2 = None
        actual_angle2 = Line.angle_between_two_lines(self.line, self.line4)

        self.assertEqual(expected_angle2, actual_angle2, "Actual != Expected")

    def test_point_of_intersection(self):
        expected_intersection1 = self.point3
        actual_intersection1 = Line.point_of_intersection(self.line2, self.line3)

        self.assertEqual(expected_intersection1, actual_intersection1, "Actual != Expected")

        expected_intersection2 = Vector2(None, None)
        actual_intersection2 = Line.point_of_intersection(self.line, self.line4)

        self.assertEqual(expected_intersection2, actual_intersection2, "Actual != Expected")

    def test_intersect(self):
        expected_intersect1 = True
        actual_intersect1 = Line.intersect(self.line2, self.line3)

        self.assertEqual(expected_intersect1, actual_intersect1, "Actual != Expected")

        expected_intersect2 = False
        actual_intersect2 = Line.intersect(self.line, self.line4)

        self.assertEqual(expected_intersect2, actual_intersect2, "Actual != Expected")

    def test_eq(self):
        expected_eq1 = False
        actual_eq1 = self.line2 == self.line4

        self.assertEqual(expected_eq1, actual_eq1, "Actual != Expected")

        expected_eq2 = True
        actual_eq2 = self.line3 == self.line5

        self.assertEqual(expected_eq2, actual_eq2, "Actual != Expected")


if __name__ == '__main__':
    unittest.main()
