import unittest
from math import isclose  # Floating point is imprecise
from ft_euler.geometry.Vector2 import Vector2
from ft_euler.geometry.Triangle import Triangle
from ft_euler.geometry.Line import Line

class TestTriangle(unittest.TestCase):
    def setUp(self):
        self.point1 = Vector2(0, 0)
        self.point2 = Vector2(3, 0)
        self.point3 = Vector2(0, 4)
        
        self.side1 = Line(self.point1, self.point2)
        self.side2 = Line(self.point2, self.point3)
        self.side3 = Line(self.point3, self.point1)

        self.triangle = Triangle(self.point1, self.point2, self.point3)

    
    def test_to_sides(self):
        expected_to_sides = [self.side1, self.side2, self.side3]
        actual_to_sides = self.triangle.to_sides()
        
        self.assertEqual(expected_to_sides, actual_to_sides, "Actual != Expected")
        

    def test_calculate_side_lengths(self):
        expected_side_lengths = [3.0, 5.0, 4.0]
        actual_side_lengths = self.triangle.calculate_side_lengths()

        expected_side_lengths.sort()
        actual_side_lengths.sort()

        for expected_side_length, actual_side_length in zip(expected_side_lengths, actual_side_lengths):
            self.assertTrue(isclose(expected_side_length, actual_side_length))

    def test_calculate_area(self):
        expected_area = 6.0
        calculated_area = self.triangle.calculate_area()

        self.assertTrue(isclose(expected_area, calculated_area))


if __name__ == '__main__':
    unittest.main()
