import unittest
from math import isclose  # Floating point is imprecise
from ft_euler.geometry.Vector2 import Vector2


class TestVector2(unittest.TestCase):
    def setUp(self):
        self.v1 = Vector2(0, 0)
        self.v2 = Vector2(3, 4)
        
        self.float_list = [3, 4]
    
    def test_from_float_list(self):
        expected_from_float_list = self.v2
        actual_from_float_list = Vector2.from_float_list(self.float_list)
        
        self.assertEqual(expected_from_float_list, actual_from_float_list, "Acutal != Expected")
        
    def test_to_float_list(self):
        expected_to_float_list = self.float_list
        actual_to_float_list = self.v2.to_float_list()
    
        self.assertEqual(expected_to_float_list, actual_to_float_list, "Acutal != Expected")

    def test_distance(self):
        expected_distance = 5.0
        actual_distance = Vector2.distance(self.v1, self.v2)

        self.assertTrue(isclose(expected_distance, actual_distance))

    def test_midpoint(self):
        expected_midpoint = Vector2(1.5, 2)
        actual_midpoint = Vector2.midpoint(self.v1, self.v2)

        self.assertTrue(isclose(expected_midpoint.x, actual_midpoint.x))
        self.assertTrue(isclose(expected_midpoint.y, actual_midpoint.y))

    def test_eq(self):
        self.assertTrue(self.v1 == self.v1)
        self.assertFalse(self.v1 == self.v2)


if __name__ == '__main__':
    unittest.main()
