import rclpy
from rclpy.node import Node
from eufs_msgs.msg import ConeWithCovariance, ConeArrayWithCovariance, CarState, FullState, CanState, ConeArray, PointArray
from geometry_msgs.msg import TwistWithCovarianceStamped, Point
from std_msgs.msg import Header
import numpy as np
import math
from sklearn.cluster import DBSCAN
import random
from collections import Counter

subscribe_msg_type = ConeArrayWithCovariance
sub_topic = "/map"
pub_topic = "/map/test"


memory = np.array([None, None, None])

class virtual_cones(Node):
    
    
    def __init__(self):
        super().__init__("check_frame")

        self.cone_sub = self.create_subscription(subscribe_msg_type, sub_topic, self.create_virtual, 10)
        self.virtual_cones = self.create_publisher(subscribe_msg_type, pub_topic, 10)
    

    
    def create_virtual(self, data):
        cones = []
        colors = []
        for cone in data.blue_cones:
            x = cone.point.x
            y = cone.point.y
            cones.append((x, y))
            colors.append(1)

        for cone in data.yellow_cones:
            x = cone.point.x
            y = cone.point.y
            cones.append((x, y))
            colors.append(2)
            
        point1 = None
        point2 = None
        virtual_point = []
        virtual_point_color = []
        
        if len(cones) >= 4:
            for cone in range(len(cones)-1, 0, -1):
                if colors[cone] == 2:
                    point1 = cones[cone]
                    for other in range(len(cones)-1, 0, -1):
                        if colors[other] == 1:
                            point2 = cones[other]
                            break
                        else:
                            continue
                else:
                    continue
            
        if point1 is not None and point2 is not None:
            mid = Point()
            mid_x = ((point1[0] + point2[0])/2)
            mid_y = ((point1[1] + point2[1])/2)
            if random.randint(0, 100) >= 95:
                cones.append((mid_x, mid_y))
                colors.append(2)
            cone_array = np.array(cones)
            color_arr = np.array(colors)
            #self.get_logger().info(str(virtual_point))
            

        msg = ConeArrayWithCovariance(header=Header(frame_id="base_footprint"))
        for idx, cone in enumerate(cones):
            cone_msg = ConeWithCovariance()
            cone_msg.point.x, cone_msg.point.y = cone
            if colors[idx] == 1:
                msg.blue_cones.append(cone_msg)
            if colors[idx] == 2:
                msg.yellow_cones.append(cone_msg)

        if virtual_point_color == 1:
            msg.blue_cones[-1].covariance = [4.0,2.0,2.0,4.0]
            
        if virtual_point_color == 2:
            
            msg.blue_cones[-1].covariance = [4.0,2.0,2.0,4.0]
        # self.get_logger().info(cones)
        # self.get_logger().info(cone_colors)
        self.virtual_cones.publish(msg)
        
        
        
def main(args=None):
    rclpy.init(args=args)
    check = virtual_cones()
    rclpy.spin(check)
    check.destroy_node()
    rclpy.shutdown()

if __name__ == "__main__":
    main()
